\name{DublinWind}
\alias{DublinWind}
\non_function{}
\title{Daily Wind Speeds in Dublin}
\usage{data(DublinWind)}
\description{
  The \code{DublinWind} data frame is basically the time series of daily
  average wind speeds from 1961 to 1978, measured in Dublin, Ireland.
  These are 6574 observations (18 full years among which four leap years).
}
\format{
  This data frame contains the following columns:
  \describe{
    \item{speed}{numeric vector of average daily wind speed in knots}
    \item{day}{an integer vector giving the day number of the year,
      i.e., 1:366.}
  }
}
\details{
  The periodic pattern along the 18 years measured and the
  autocorrelation are to be taken into account for analysis, see the
  references.  This is Example 3 of the COBS paper.
}
\source{
  From shar file available from \url{http://www.cba.nau.edu/pin-ng/cobs.html}

  Also available from \url{.......STATLIB..........}
}
\references{
  Haslett, J. and Raftery, A. (1989)
  Space-Time Modelling with Long-Memory Dependence: {A}ssessing
  {I}reland's Wind Power Resource.
  .......... % FIXME -- no journal !
  \bold{38}, 1--21.

  COBS: Qualitatively Constrained Smoothing via Linear Programming;
  \emph{Computational Statistics} \bold{14}, 315--337.

  He, X. and Ng, P. (1999)
  COBS: Qualitatively Constrained Smoothing via Linear Programming;
  \emph{Computational Statistics} \bold{14}, 315--337.
}
\examples{
data(DublinWind)
str(DublinWind)
plot(speed ~ day, data = DublinWind, type = "l")
attach(DublinWind)
plot(wSpeed <- ts(speed))
%%## FIXME: use correct time axis!--> need leap year correctness
%%##wSpeed <- ts(speed, start = 1961, end = 1978.999, periodicity= (365 + 1/4))

%% currently He & Ng "Example 3" is --> ../tests/wind.R
}
\keyword{datasets}
\eof
\name{USArmyRoofs}
\alias{USArmyRoofs}
\non_function{}
\title{Roof Quality in US Army Bases}
\usage{data(USArmyRoofs)}
\description{
  The \code{USArmyRoofs} data frame has 153 observations of roof sections of US
  Army bases and 2 columns, \code{age} and \code{fci}.  This is Example 2
  of He & Ng (1999).
}
\format{
  This data frame contains the following columns:
  \describe{
    \item{age}{numeric vector giving the roof's age in years.}
    \item{fci}{numeric, giving the FCI, the flash condition index, i.e.,
      the percentage of flashing which is in good condition.}
  }
}
\source{
  From shar file available from \url{http://www.cba.nau.edu/pin-ng/cobs.html}
}
\references{
  He, X. and Ng, P. (1999)
  COBS: Qualitatively Constrained Smoothing via Linear Programming;
  \emph{Computational Statistics} \bold{14}, 315--337.
}
\examples{
data(USArmyRoofs)
plot(fci ~ age, data = USArmyRoofs, main = "US Army Roofs data")

%% currently He & Ng "Example 2" is --> ../tests/roof.R
}
\keyword{datasets}
\eof
\name{cobs-internal}
\title{Internal COBS functions}
%      -----------------------
%%-- maybe more of these really should be documented and ``exported''
\alias{getdim}
\alias{l1.design}
\alias{loo.design}
\alias{dn}% two simple bandwidth rules
\alias{shat}% sparsity estimate via resid.
\alias{.splBasis}
\alias{.splValue}

\alias{cut00}
%
\description{
  Internal cobs functions.
}
\usage{
dn(p, n, hs = FALSE, alpha)

getdim(degree, nknots, constraint)

l1.design(x, w, constraint, equal, smaller, greater, gradient,
          knots, pw, n.equal, n.smaller, n.greater, n.gradient, nrq,
          nl1, neqc, niqc, nvar, lambda)
loo.design(x, w, constraint, equal, smaller, greater, gradient,
          knots, pw, n.equal, n.smaller, n.greater, n.gradient, nrq,
          nl1, neqc, niqc, nvar, lambda)

shat(residual, tau, alpha, hs)

.splValue(degree, knots, coef, xo)
.splBasis(ord, knots, ncoef, xo, derivs)
}
\details{
  These are not (yet?) to be called by the user and have not been
  documented by the original COBS authors.
}
\keyword{internal}
\eof
\name{cobs-methods}
\alias{print.cobs}
\alias{summary.cobs}
\alias{fitted.cobs}
\alias{residuals.cobs}
\title{Methods for COBS Objects}
\description{
  Print, summary and other methods for \code{\link{cobs}} objects.
}
\usage{
\method{print}{cobs}(x, digits = getOption("digits"), \dots)
\method{summary}{cobs}(object, digits = getOption("digits"), \dots)

\method{fitted}{cobs}(object, \dots)
\method{residuals}{cobs}(object, \dots)

}
\arguments{
  \item{x, object}{object of class \code{cobs}.}
  \item{digits}{number of digits to use for printing.}
  \item{\dots}{further arguments passed from and to methods.}
}
\value{
  \code{print.cobs()} returns its argument invisibly.
}
\details{
  These are methods for fitted COBS objects, as computed by
  \code{\link{cobs}}.
}
\author{Martin Maechler}
\seealso{
  \code{\link{predict.cobs}} for the \code{\link{predict}} method, and
  \code{\link{cobs}} for examples.
}
\examples{
example(cobs)% ./cobs.Rd
summary(Sbs)

plot(fitted(Sbs), resid(Sbs), main = deparse(Sbs$call))
}
\keyword{print}
\eof
\name{cobs}
\alias{cobs}
\alias{n1000cut}
\title{COnstrained B-Splines Nonparametric Regression Quantiles}
\description{
  Computes constrained quantile curves using linear or
  quadratic splines.  The median spline (\eqn{L_1} loss) is a robust
  (constrained) smoother.
}
\usage{
cobs(x, y, constraint = c("none", "increase", "decrease",
                          "convex", "concave", "periodic"),
     knots, nknots, method = "quantile",
     degree = 2, tau = 0.5, lambda = 0, ic = "aic",
     n.sub = n1000cut(n),
     knots.add = FALSE, pointwise,
     print.warn = TRUE, print.mesg = TRUE, trace = print.mesg,
     coef = rep(0,nvar), w = rep(1,n),
     maxiter = 20*n, lstart = 7872, toler.kn = 1e-6,
     eps = .Machine$double.eps, factor = 1)

n1000cut(n)
}
\arguments{
  \item{x}{vector of covariate; missing values are omitted.}
  \item{y}{vector of response variable.  It must have the same length as
    \code{x}.}
  \item{constraint}{character (string) specifying the kind of
    constraint; must be one of the values in the default list, above;
    may be abbreviated.}
  \item{knots}{vector of locations of the knot mesh; if missing,
    \code{nknots} number of \code{knots} will be created using the
    specified \code{method} and automatic knot selection will be carried
    out for regression B-spline (lambda=0); if not missing and
    \code{length(knots)==nknots}, the provided knot mesh will be used in
    the fit and no automatic knot selection will be performed;
    otherwise, automatic knots selection will be performed on the
    provided \code{knots}.}
  \item{nknots}{maximum number of knots; defaults to 6 for regression
    B-spline, 20 for smoothing B-spline.}
  \item{method}{character specifying the method for generating
    \code{nknots} number of \code{knots} when \code{knots} is not provided;
    "quantile" (equally spaced in percentile levels)
    or "uniform" (equally spaced knots); defaults to "quantile".}
  \item{degree}{degree of the splines; 1 for linear spline and 2 for quadratic
    spline; defaults to 2.}
  \item{tau}{desired quantile level; defaults to 0.5 (median).}
  \item{lambda}{penalty parameter;
    lambda = 0: no penalty (regression B-spline);\cr
    lambda > 0: smoothing B-spline with the given lambda;\cr
    lambda < 0: smoothing B-spline with lambda chosen by an information
    criterion, see \code{ic}.}
  \item{ic}{information criterion used in knot deletion and addition for
    regression B-spline method when lambda=0; "aic" (Akaike-type) or
    "sic" (Schwarz-type); default to "aic".}
  \item{n.sub}{integer, not larger than sample size \code{n}; the
    default has \code{n.sub == n} as long as \code{n} is less than 1000.}
  \item{knots.add}{logical indicating if an additional step of stepwise
    knot addition should be performed for regression B-splines.}
  \item{pointwise}{an optional three-column matrix with each row
    specifies one of the following constraints:
    \describe{
      \item{\code{( 1,xi,yi)}:}{fitted value at xi will be \eqn{\ge}{>=} yi;}
      \item{\code{(-1,xi,yi)}:}{fitted value at xi will be \eqn{\le}{<=} yi;}
      \item{\code{( 0,xi,yi)}:}{fitted value at xi will be \eqn{=} yi;}
      \item{\code{( 2,xi,yi)}:}{derivative of the fitted function at xi
	will be yi.}
      }
  }
  \item{print.warn}{
    logical flag for printing of interactive warning messages; true by default;
    probably needs to be set to \code{FALSE} if performing simulation.}
  \item{print.mesg}{
    logical flag or integer for printing of intermediate messages; true
    by default.  Probably needs to be set to \code{FALSE} in simulations.}
  \item{trace}{integer \eqn{\ge 0}{>= 0} indicating how much the Fortran
    routine \code{drqssbc} should print intermediate messages;
    defaults to \code{print.mesg}.}
  \item{coef}{initial guess of the B-spline coefficients; default to a
    vector of zeros.}
  \item{w}{
    vector of weights the same length as x (y) assigned to both x and y;
    default to uniform weights adding up to one; using normalized weights
    that add up to one will speed up computation.
  }
  \item{maxiter}{upper bound of the number of iteration; default to 20*n.}
  \item{lstart}{starting value for lambda when performing parametric
    programming in lambda if \code{lambda < 0}; defaults to
    \code{log(big)^2}.}
  \item{toler.kn}{numeric tolerance for shifting the boundary knots
    outside; default 1e-6 used to be built in.}
  \item{eps}{tolerance passed to \code{\link{qbsks}} and \code{\link{drqssbc}}.}
  \item{factor}{
    determines how big a step to the next smaller lambda should be while
    performing parametric programming in lambda; the default \code{1}
    will give all unique lambda's; use of a bigger factor \eqn{(> 1 &  < 4)}
    will save time for big problems.}
  %
  \item{n}{integer, the sample size.}
}
\value{
  an object of class \code{cobs}, a list with components
  \item{call}{the \code{cobs(..)} call used for creation.}
  \item{tau, degree}{as input}
  \item{constraint}{as input (but no more abbreviated).}
  \item{call}{the \code{cobs(..)} call used for creation.}
  \item{coef}{B-spline coefficients.}
  \item{knots}{the final set of knots used in the computation.}
  \item{ifl}{exit code:
    \describe{
      \item{1}{-- ok;}
      \item{2}{-- problem is infeasible, check specification of the
	\code{pointwise} argument;}
      \item{3}{-- maxiter is reached before finding a solution,
	either increase maxiter and restart the program with \code{coef}
	set to the value upon previous exit or use a smaller
	\code{lstart} value when lambda<0 or use a smaller \code{lambda}
	value when lambda>0;}
      \item{4}{-- program aborted, numerical difficulties due to
	ill-conditioning.}
    }
  }
  \item{icyc}{length 2: number of cycles taken to achieve convergence
    for final lambda, and total number of cycles for all lambdas.}
  \item{k}{the effective dimensionality of the final fit.}
  \item{k0}{(usually the same)}
  \item{x.ps}{the pseudo design matrix \eqn{X} (as returned by
    \code{\link{qbsks}}).}
  \item{resid}{vector of residuals from the fit.}
  \item{fitted}{vector of fitted values from the fit.}
  \item{SSy}{the sum of squares around centered \code{y} (e.g. for
    computation of \eqn{R^2}.)}
  \item{lambda}{the penalty parameter used in the final fit.}
  \item{pp.lambda}{vector of all unique lambda's obtained from
    parametric programming when \code{lambda} < 0 on input.}
  \item{sic}{vector of Schwarz information criteria evaluated at
    \code{pp.lambda}.}
}%end{value}
\details{
  \code{cobs()} computes the constraint quantile smoothing B-spline with
  penalty when lambda is not zero.\cr
  If lambda < 0, an optimal lambda will be chosen using Schwarz type
  information criterion. \cr
  If lambda > 0, the supplied lambda will be used.\cr
  If lambda = 0, cobs computes the constraint quantile regression B-spline
  with no penalty using the provided knots or those selected by Akaike or
  Schwarz information criterion.
}
\references{
  He, X. and Ng, P. (1999)
  COBS: Qualitatively Constrained Smoothing via Linear Programming;
  \emph{Computational Statistics} \bold{14}, 315--337.

  Koenker, R. and Ng, P. (1996)
  A Remark on Bartels and Conn's Linearly Constrained L1 Algorithm,
  \emph{ACM Transaction on Mathematical Software} \bold{22}, 493--495.

  Ng, P. (1996)
  An Algorithm for Quantile Smoothing Splines,
  \emph{Computational Statistics & Data Analysis} \bold{22}, 99--118.

  Bartels, R. and Conn A. (1980)
  Linearly Constrained Discrete \eqn{L_1} Problems,
  \emph{ACM Transaction on Mathematical Software} \bold{6}, 594--608.

  A postscript version of the paper that describes the details of COBS
  can be downloaded from \url{http://www.cba.nau.edu/pin-ng/cobs.html}
}
\section{Warning}{
  This is still a beta version, and we do appreciate comments and
  suggestions; \code{library(help = cobs)} shows the authors.
}
\seealso{\code{\link[modreg]{smooth.spline}} for unconstrained smoothing
    splines; \code{\link[splines]{bs}} for unconstrained (regression)
    B-splines.}
}
\examples{
x <- seq(-1,1,,50)
y <- (f.true <- pnorm(2*x)) + rnorm(50)/10
## specify pointwise constraints (boundary conditions)
con <- rbind(c( 1,min(x),0), # f(min(x)) >= 0
             c(-1,max(x),1), # f(max(x)) <= 1
             c(0,  0,   0.5))# f(0)      = 0.5

## obtain the median regression B-spline using automatically selected knots
Rbs <- cobs(x,y,constraint="increase",pointwise=con)
Rbs

plot(x,y)
lines(predict(Rbs), col = 2, lwd = 1.5)
lines(spline(x,f.true), col = "gray40")

Rbsub <- cobs(x,y,constraint="increase",pointwise=con, n.sub = 45)
%%--> Current newly introduced BUG -- ../R/cobs.R.~21~ *does* work
summary(Rbsub)
lines(predict(Rbsub), col = 4, lwd = 1)


## compute the median smoothing B-spline using automatically chosen lambda
Sbs <- cobs(x,y,constraint="increase",pointwise=con,lambda=-1)
Sbs
plot(Sbs$pp.lambda[-1], Sbs$sic[-1], log = "x",
     main = "SIC ~ lambda", xlab = expression(lambda), ylab = "SIC")
axis(1, at = Sbs$lambda, label = expression(hat(lambda)),
     col.axis = 2, mgp = c(3, 0.5, 0))

Sb1 <- cobs(x,y,constraint="increase",pointwise=con,lambda=-1, degree=1)
summary(Sb1)
pxx <- predict(Sb1, xx <- seq(-1.2, 1.2, len = 201), interval = "both")
plot(x,y, main = deparse(Sb1$call),
     xlim = range(xx), ylim = range(y, pxx[,"fit"]))
lines(pxx, col = 2)
rug(Sb1$knots, col = 4, lwd = 1.6)# (too many knots)
matlines(pxx[,1], pxx[,-(1:2)], col= rep(c("green","blue"),c(2,2)), lty=2)
}
% Converted by Sd2Rd version 1.21.
\keyword{smooth}
\keyword{regression}
\eof
\name{cobsOld}
\alias{cobsOld}
\title{COnstrained B-Splines Nonparametric Regression Quantiles}
}
\description{
  Computes constrained quantile curves using linear or
  quadratic splines.  The median spline (\eqn{L_1} loss) is a robust
  (constrained) smoother.
}
\usage{
cobsOld(x, y, constraint = c("none", "increase", "decrease",
                          "convex", "concave", "periodic"),
     z, minz = knots[1], maxz = knots[nknots], nz = 100,
     knots, nknots, method = "quantile",
     degree = 2, tau = 0.5, lambda = 0, ic = "aic",
     knots.add = FALSE, alpha = 0.1, pointwise,
     print.warn = TRUE, print.mesg = TRUE, trace = print.mesg,
     coef = rep(0,nvar), w = rep(1,n),
     maxiter = 20*n, lstart = log(big)^2, toler = 1e-6,
     factor = 1)
}
\arguments{
  \item{x}{vector of covariate.}
  \item{y}{vector of response variable. It must have the same length as x.}
  \item{constraint}{character (string) specifying the kind of constraint;
    must be one of "increase", "decrease", "convex", "concave",
    "periodic" or "none".}
  \item{z}{vector of grid points at which the fitted values are
    evaluated; default to an equally spaced grid with \code{nz} grid
    points between \code{minz} and \code{maxz}.  If the fitted values at
    x are desired, use \code{z = unique(x)}.}
  \item{minz}{numeric needed if \code{z} is not specified; defaults to
    \code{min(x)} or the first knot if \code{knots} are given.}
  \item{maxz}{analogous to \code{minz}; defaults to \code{max(x)} or the
    last knot if \code{knots} are given.}
  \item{nz}{number of grid points in \code{z} if that is not given;
    defaults to 100.}
  \item{knots}{vector of locations of the knot mesh; if missing,
    \code{nknots} number of \code{knots} will be created using the
    specified \code{method} and automatic knot selection will be carried
    out for regression B-spline (lambda=0); if not missing and
    \code{length(knots)==nknots}, the provided knot mesh will be used in
    the fit and no automatic knot selection will be performed;
    otherwise, automatic knots selection will be performed on the
    provided \code{knots}.}
  \item{nknots}{maximum number of knots; defaults to 6 for regression
    B-spline, 10 for smoothing B-spline.}
  \item{method}{character specifying the method for generating
    \code{nknots} number of \code{knots} when \code{knots} is not provided;
    "quantile" (equally spaced in percentile levels)
    or "uniform" (equally spaced knots); defaults to "quantile".}
  \item{degree}{degree of the splines; 1 for linear spline and 2 for quadratic
    spline; defaults to 2.}
  \item{tau}{desired quantile level; defaults to 0.5 (median).}
  \item{lambda}{penalty parameter;
    lambda = 0: no penalty (regression B-spline);\cr
    lambda > 0: smoothing B-spline with the given lambda;\cr
    lambda < 0: smoothing B-spline with lambda chosen by a Schwarz-type
    information criterion.}
  \item{ic}{information criterion used in knot deletion and addition for
    regression B-spline method when lambda=0; "aic" (Akaike-type) or
    "sic" (Schwarz-type); default to "aic".}
  \item{knots.add}{logical indicating if an additional step of stepwise
    knot addition should be performed for regression B-splines.}
  \item{alpha}{level of significance for the confidence band.}
  \item{pointwise}{an optional three-column matrix with each row
    specifies one of the following constraints:
    \describe{
      \item{\code{( 1,xi,yi)}:}{fitted value at xi will be \eqn{\ge}{>=} yi;}
      \item{\code{(-1,xi,yi)}:}{fitted value at xi will be \eqn{\le}{<=} yi;}
      \item{\code{( 0,xi,yi)}:}{fitted value at xi will be \eqn{=} yi;}
      \item{\code{( 2,xi,yi)}:}{derivative of the fitted function at xi
	will be yi.}
      }
  }
  \item{print.warn}{
    logical flag for printing of interactive warning messages; default to T;
    probably needs to be set to F if performing monte carlo simulation.
  }
  \item{print.mesg}{
    logical flag for printing of intermediate messages; default to T;
    probably needs to be set to F if performing monte carlo simulation.
  }
  \item{trace}{integer \eqn{\ge 0}{>= 0} indicating how much the Fortran
    routine \code{drqssbc} should print intermediate messages;
    defaults to \code{print.mesg}, i.e. 1 (or 0).}
  \item{coef}{initial guess of the B-spline coefficients; default to a
    vector of zeros.}
  \item{w}{
    vector of weights the same length as x (y) assigned to both x and y;
    default to uniform weights adding up to one; using normalized weights
    that add up to one will speed up computation.
  }
  \item{maxiter}{upper bound of the number of iteration; default to 20*n.}
  \item{lstart}{starting value for lambda when performing parametric
    programming in lambda if \code{lambda < 0}; defaults to
    \code{log(big)^2}.}
  \item{toler}{numeric tolerance for ???? ; default 1e-6 used to be builtin.}
  \item{factor}{
    determines how big a step to the next smaller lambda should be while
    performing parametric programming in lambda; the default \code{1}
    will give all unique lambda's; use of a bigger factor \eqn{(> 1 &  < 4)}
    will save time for big problems.}
}
\value{
  a list with components
  \item{coef}{B-spline coefficients.}
  \item{fit}{fitted value at z.}
  \item{resid}{vector of residuals from the fit.}
  \item{z}{as in input.}
  \item{knots}{the final set of knots used in the computation.}
  \item{ifl}{exit code:
    \describe{
      \item{1}{-- ok;}
      \item{2}{-- problem is infeasible, check specification of the
	\code{pointwise} argument;}
      \item{3}{-- maxiter is reached before finding a solution,
	either increase maxiter and restart the program with \code{coef}
	set to the value upon previous exit or use a smaller
	\code{lstart} value when lambda<0 or use a smaller \code{lambda}
	value when lambda>0;}
      \item{4}{-- program aborted, numerical difficulties due to
	ill-conditioning.}
    }
  }
  \item{icyc}{number of cycles taken to achieve convergence.}
  \item{k}{the effective dimensionality of the final fit.}
  \item{lambda}{the penalty parameter used in the final fit.}
  \item{pp.lambda}{vector of all unique lambda's obtained from
    parametric programming when lambda < 0 on input.}
  \item{sic}{vector of Schwarz information criteria evaluated at pp.lambda.}
  \item{cb.lo}{lower bound of the confidence band}
  \item{cb.up}{upper bound of the confidence band}
  \item{ci.lo}{lower bound of the pointwise confidence interval}
  \item{ci.up}{upper bound of the pointwise confidence interval}
}%end{value}
\details{
  \code{cobsOld()} computes the constraint quantile smoothing B-spline with
  penalty when lambda is not zero.\cr
  If lambda < 0, an optimal lambda will be chosen using Schwarz type
  information criterion. \cr
  If lambda > 0, the supplied lambda will be used.\cr
  If lambda = 0, cobsOld computes the constraint quantile regression B-spline
  with no penalty using the provided knots or those selected by Akaike or
  Schwarz information criterion.
}
\references{
  He, X. and Ng, P. (1999)
  COBS: Qualitatively Constrained Smoothing via Linear Programming;
  \emph{Computational Statistics} \bold{14}, 315--337.

  Koenker, R. and Ng, P. (1996)
  A Remark on Bartels and Conn's Linearly Constrained L1 Algorithm,
  \emph{ACM Transaction on Mathematical Software} \bold{22}, 493--495.

  Ng, P. (1996)
  An Algorithm for Quantile Smoothing Splines,
  \emph{Computational Statistics & Data Analysis} \bold{22}, 99--118.

  Bartels, R. and Conn A. (1980)
  Linearly Constrained Discrete \eqn{L_1} Problems,
  \emph{ACM Transaction on Mathematical Software} \bold{6}, 594--608.

  A postscript version of the paper that describes the details of COBS
  can be downloaded from \url{http://www.cba.nau.edu/pin-ng/cobs.html}
}
\section{Warning}{
  This is still a beta version, and we do appreciate comments and
  suggestions; \code{library(help = cobs)} shows the authors.
}
\seealso{\code{\link[modreg]{smooth.spline}} for unconstrained smoothing
    splines; \code{\link[splines]{bs}} for unconstrained (regression)
    B-splines.}
}
\examples{
x <- seq(-1,1,,50)
y <- (f.true <- pnorm(2*x)) + rnorm(50)/10
## specify pointwise constraints (boundary conditions)
con <- rbind(c( 1,min(x),0), # f(min(x)) >= 0
             c(-1,max(x),1), # f(max(x)) <= 1
             c(0,  0,   0.5))# f(0)      = 0.5

## obtain the median regression B-spline using automatically selected knots
cobsOld(x,y,constraint="increase",pointwise=con)->cobs.o

plot(x,y)
lines(cobs.o$z, cobs.o$fit, col = 2, lwd = 1.5)
lines(spline(x,f.true), col = "gray40")
lines(cobs.o$z, cobs.o$cb.lo,lty=2, col = 3)
lines(cobs.o$z, cobs.o$cb.up,lty=2, col = 3)

## compute the median smoothing B-spline using automatically chosen lambda
cobsOld(x,y,constraint="increase",pointwise=con,lambda=-1)->cobs.oo
plot(x,y, main = "COBS Median smoothing spline, automatical lambda")
lines(spline(x,f.true), col = "gray40")
lines(cobs.oo$z, cobs.oo$fit)
lines(cobs.oo$z, cobs.oo$cb.lo,lty=2)
lines(cobs.oo$z, cobs.oo$cb.up,lty=2)

plot(cobs.oo$pp.lambda[-1], cobs.oo$sic[-1], log = "x",
     main = "SIC ~ lambda", xlab = expression(lambda), ylab = "SIC")
axis(1, at = cobs.oo$lambda, label = expression(hat(lambda)),
     col.axis = 2, mgp = c(3, 0.5, 0))
}
% Converted by Sd2Rd version 1.21.
\keyword{smooth}
\keyword{regression}
\eof
\name{drqssbc}
\alias{drqssbc}
\title{Regression Quantile Smoothing Spline with Constraints}
\description{
  Estimate the B-spline coefficients for a regression quantile
  \emph{smoothing} spline with optional constraints, using Ng(1996)'s
  algorithm.
}
\usage{
drqssbc(x, y, w= rep(1,n), pw, knots, degree, Tlambda, constraint,
        n.sub = n1000cut(nrq),
        equal, smaller, greater, gradient, coef, maxiter = 20 * n,
        trace = 1,
        n.equal = nrow(equal), n.smaller = nrow(smaller),
        n.greater = nrow(greater), n.gradient = nrow(gradient),
        nrq = length(x), nl1, neqc, niqc, nvar, nj0,
        tau = 0.5, lam, tmin, kmax, lstart, factor,
        eps = .Machine$double.eps, print.warn)
}
\arguments{
  \item{x}{numeric vector, sorted increasingly, the abscissa values}
  \item{y}{numeric, same length as \code{x}, the observations.}
  \item{w}{numeric vector of weights, same length as \code{x}, as in
    \code{\link{cobs}}.}
  \item{pw}{penalty weights vector passed to \code{\link{l1.design}} or
    \code{\link{loo.design}}.}
  \item{knots}{ ~~Describe \code{knots} here~~ }
  \item{degree}{integer, must be 1 or 2.}
  \item{Tlambda}{ ~~Describe \code{Tlambda} here~~ }
  \item{constraint}{see \code{\link{cobs}} (but cannot be abbreviated here).}
  \item{n.sub}{integer, not larger than sample size \code{n}; the
    default has \code{n.sub == n} as long as \code{n} is less than 1000.}
  \item{equal,smaller, greater}{3-column matrices specifying the
    respective constraints.  The has 0 zeros if there no constraints of
    the corresponding kind.}
  \item{gradient}{3-column matrix for gradient constraints.}
  \item{coef}{numeric vector, the initial guess for the B-spline coefficients.}
  \item{maxiter}{upper bound of the number of iteration; default to 20*n.}
  \item{trace}{ ~~Describe \code{trace} here~~ }
  \item{n.equal,n.smaller,n.greater,n.gradient}{
    ~~Describe \code{n.gradient} here~~ }
  \item{nrq}{ ~~Describe \code{nrq} here~~ }
  \item{nl1}{ ~~Describe \code{nl1} here~~ }
  \item{neqc}{integer giving the number of equations.}
  \item{niqc}{integer giving the number of \bold{i}ne\bold{q}uality
    \bold{c}onstraints.}
  \item{nvar}{integer giving the number of equations \emph{and} constraints.}
  \item{nj0}{ ~~Describe \code{nj0} here~~ }
  \item{tau}{desired quantile level; defaults to 0.5 (median).}
  \item{lam}{ ~~Describe \code{lam} here~~ }
  \item{tmin}{ ~~Describe \code{tmin} here~~ }
  \item{kmax}{ ~~Describe \code{kmax} here~~ }
  \item{lstart}{number, see \code{\link{cobs}.}}
  \item{factor}{number in [1,4], see \code{\link{cobs}}.}
  \item{eps}{tolerance used in the fortran code in many different contexts.}
  \item{print.warn}{logical indicating if warnings should be printed,
    when the algorithm seems to have behaved somewhat unexpectedly.}
}
\details{
  This is an auxiliary function for \code{\link{cobs}}, possibly
  interesting on its own.  This documentation is currently sparse; read
  the source code!
}
\value{
  a list with components
  \item{comp1 }{Description of `comp1'}
  \item{comp2 }{Description of `comp2'}
  ...
}
\references{
  Ng, P. (1996)
  An Algorithm for Quantile Smoothing Splines,
  \emph{Computational Statistics \& Data Analysis} \bold{22}, 99--118.
}
\author{Pin Ng; this help page: Martin Maechler.}
\seealso{
  the main function \code{\link{cobs}} and its auxiliary
  \code{\link{qbsks}} which calls \code{drqssbc()} repeatedly.
}
\examples{
set.seed(1243)
x  <- 1:32
fx <- (x-5)*(x-15)^2*(x-21)
y  <- fx + round(rnorm(x,s = 0.25),2)
## FAILS  drqssbc(x,y,nrq=32,lam=1,degree=1,knots=c(1,5,15,32))
}
\keyword{smooth}
\keyword{utilities}
\eof
\name{exHe}
\alias{exHe}
\non_function{}
\title{Small Dataset Example of He}
\usage{data(exHe)}
\description{
  The \code{exHe} data frame has 10 rows and 2 columns.  It is an
  example for which \code{\link[modreg]{smooth.spline}} cannot be used.
}
\format{
  This data frame contains the following columns:
  \describe{
    \item{x}{only values 0, 1, and 2.}
    \item{y}{10 randomly generated values}
  }
}
\source{
  Found at the bottom of \url{http://ux6.cso.uiuc.edu/~x-he/ftp.html},
  signed by \email{x-he@uiuc.edu}.
}
\examples{
data(exHe)
plot(exHe, main = "He's 10 point example and cobs() fits")
tm <- tapply(exHe$y, exHe$x, mean)
lines(unique(exHe$x), tm, lty = 2)
## 4 warnings :
cH  <- cobsOld(exHe$x, exHe$y, constraint = "increase")
lines(cH$z, cH$fit, col = 3)
cHn <- cobsOld(exHe$x, exHe$y, constraint = "none")
lines(cHn$z, cHn$fit, col = 4)
cHd <- cobsOld(exHe$x, exHe$y, constraint = "decrease")
lines(cHd$z, cHd$fit, col = 5)
}
\keyword{datasets}
\eof
\name{globtemp}
\alias{globtemp}
\non_function{}
\title{Annual Average Global Surface Temperature}
\usage{data(globtemp)}
\description{
  Time Series of length 113 of annual average global surface temperature
  deviations from 1880 to 1992.
%% deviations from what?  The mean is not 0, but -0.084; median = -0.06
}
\details{
  This is Example 1 of the COBS paper, where the hypothesis of a
  monotonely increasing trend is considered; Koenker and Schorfheide
  (1994) consider modeling the autocorrelations.
}
\source{
  \file{temp.data} in file \file{cobs.shar} available from
  \url{http://www.cba.nau.edu/pin-ng/cobs.html}.
}
\references{
  He, X. and Ng, P. (1999)
  COBS: Qualitatively Constrained Smoothing via Linear Programming;
  \emph{Computational Statistics} \bold{14}, 315--337.

  Koenker, R. and Schorfheide F. (1994)
  Quantile Spline Models for Global Temperature Change;
  \emph{Climate Change} \bold{28}, 395--404.
}
\examples{
data(globtemp)
plot(globtemp, main = "Annual Global Temperature Deviations")

%% currently He & Ng "Example 1" is --> ../tests/temp.R
}
\keyword{datasets}
\eof
\name{predict.cobs}
\alias{predict.cobs}
\title{Predict method for COBS Fits}
\description{
  Compute predicted values and simultaneous or pointwise confidence
  bounds for \code{\link{cobs}} objects.
}
\usage{
\method{predict}{cobs}(object,
        z, minz = knots[1], maxz = knots[nknots], nz = 100,
        interval = c("none", "confidence", "simultaneous", "both"),
        level = 0.95, \dots)
}
\arguments{
  \item{object}{object of class \code{cobs}.}
  \item{z}{vector of grid points at which the fitted values are
    evaluated; default to an equally spaced grid with \code{nz} grid
    points between \code{minz} and \code{maxz}.  Note that now \code{z}
    may lie outside of the knots interval which was not allowed originally.}
  \item{minz}{numeric needed if \code{z} is not specified; defaults to
    \code{min(x)} or the first knot if \code{knots} are given.}
  \item{maxz}{analogous to \code{minz}; defaults to \code{max(x)} or the
    last knot if \code{knots} are given.}
  \item{nz}{number of grid points in \code{z} if that is not given;
    defaults to 100.}
  \item{interval}{type of interval calculation, see below}
  \item{level}{confidence level}
  \item{\dots}{further arguments passed to and from methods.}
}
\details{

}
\value{
  \code{predict.cobs} produces aa matrix of
  predictions and bounds if    \code{interval} is set (not "none").
  The columns are named \code{z}, \code{fit}, further \code{cb.lo} and
  \code{cb.up} for the \code{simultaneous} confidence band, and \code{ci.lo} and
  \code{ci.up} the pointwise \code{confidence} intervals according to
  specified \code{level}.
}
\author{Martin Maechler, based on He and Ng's code in \code{cobs()}.}
\seealso{\code{\link{cobs}} the model fitting function.}
\examples{
example(cobs) # continuing :
(pRbs <- predict(Rbs))
str(pSbs <- predict(Sbs, xx, interval = "both"))

plot(x,y, xlim = range(xx), ylim = range(y, pSbs[,2], finite = TRUE),
     main = "COBS Median smoothing spline, automatical lambda")
lines(pSbs, col = "red")
lines(spline(x,f.true), col = "gray40")
matlines(pSbs[,1], pSbs[,-(1:2)],
         col= rep(c("green","blue"),c(2,2)), lty=2)
}
\keyword{regression}
\eof
\name{qbsks}
\alias{qbsks}
\title{Quantile B-Spline with Fixed Knots}
\description{
  Compute B-spline coefficients for regression quantile B-spline with
  stepwise knots selection and quantile B-spline with fixed knots
  \bold{regression spline}, using Ng (1996)'s algorithm.
}
\usage{
qbsks(x,y,w,pw, knots,nknots, degree,Tlambda, constraint,
      n.sub = n1000cut(n), equal,smaller, greater,gradient, coef,maxiter,
      trace, n.equal,n.smaller,n.greater,n.gradient,
      nrq,nl1, neqc, nj0, tau,lam,tmin,kmax,lstart,
      ks,mk.flag, knots.add, ic, print.mesg,
      factor, tol.kn = 1e-6, eps = .Machine$double.eps, print.warn)
}
\arguments{
  \item{x}{numeric vector, sorted increasingly, the abscissa values}
  \item{y}{numeric, same length as \code{x}, the observations.}
  \item{w}{numeric vector of weights, same length as \code{x}, as in
    \code{\link{cobs}}.}
  \item{pw}{penalty weights vector ... ...}
  \item{knots}{ ~~Describe \code{knots} here~~ }
  \item{nknots}{ ~~Describe \code{nknots} here~~ }
  \item{degree}{integer specifying polynomial degree; must be 1 or 2.}
  \item{Tlambda}{ ~~Describe \code{Tlambda} here~~ }
  \item{constraint}{ ~~Describe \code{constraint} here~~ }
  \item{n.sub}{integer, not larger than sample size \code{n}; the
    default has \code{n.sub == n} as long as \code{n} is less than 1000.}
  \item{equal}{ ~~Describe \code{equal} here~~ }
  \item{smaller}{ ~~Describe \code{smaller} here~~ }
  \item{greater}{ ~~Describe \code{greater} here~~ }
  \item{gradient}{ ~~Describe \code{gradient} here~~ }
  \item{coef}{ ~~Describe \code{coef} here~~ }
  \item{maxiter}{ ~~Describe \code{maxiter} here~~ }
  \item{trace}{ ~~Describe \code{trace} here~~ }
  \item{n.equal}{ ~~Describe \code{n.equal} here~~ }
  \item{n.smaller}{ ~~Describe \code{n.smaller} here~~ }
  \item{n.greater}{ ~~Describe \code{n.greater} here~~ }
  \item{n.gradient}{ ~~Describe \code{n.gradient} here~~ }
  \item{nrq}{ ~~Describe \code{nrq} here~~ }
  \item{nl1}{ ~~Describe \code{nl1} here~~ }
  \item{neqc}{ ~~Describe \code{neqc} here~~ }
  \item{nj0}{ ~~Describe \code{nj0} here~~ }
  \item{tau}{ ~~Describe \code{tau} here~~ }
  \item{lam}{ ~~Describe \code{lam} here~~ }
  \item{tmin}{ ~~Describe \code{tmin} here~~ }
  \item{kmax}{ ~~Describe \code{kmax} here~~ }
  \item{lstart}{ ~~Describe \code{lstart} here~~ }
  \item{ks}{ ~~Describe \code{ks} here~~ }
  \item{mk.flag}{ ~~Describe \code{mk.flag} here~~ }
  \item{knots.add}{ ~~Describe \code{knots.add} here~~ }
  \item{ic}{ ~~Describe \code{ic} here~~ }
  \item{print.mesg}{ ~~Describe \code{print.mesg} here~~ }
  \item{factor}{ ~~Describe \code{factor} here~~ }
  \item{tol.kn}{``tolerance'' for shifting the outer knots.}
  \item{eps}{tolerance passed to \code{\link{drqssbc}}.}
  \item{print.warn}{flag indicating if and how much warnings and
    information is to be printed;  currently just passed to \code{\link{drqssbc}}.}
}
\details{
  This is an auxiliary function for \code{\link{cobs}(*, lambda = 0)},
  possibly interesting on its own.  This documentation is currently sparse; read
  the source code!
}
\value{
  a list with components
  \item{coef}{..}
  \item{fidel}{..}
  \item{k}{dimensionality of model fit.}
  \item{ifl}{integer ``flag''; the return code.}
  \item{icyc}{integer of length 2, see \code{\link{cobs}}.}
  \item{knots}{the vector of inner knots.}
  \item{nknots}{the number of inner knots.}
  \item{nvar}{the number of ``variables'', i.e. unknowns including
    constraints.}
  \item{lambda}{the penalty factor, chosen or given.}
  \item{pseudo.x}{the pseudo design matrix \eqn{X}, as returned from
    \code{\link{drqssbc}}.}
}
\references{
  Ng, P. (1996)
  An Algorithm for Quantile Smoothing Splines,
  \emph{Computational Statistics \& Data Analysis} \bold{22}, 99--118.
}
\author{Pin Ng; this help page: Martin Maechler.}

\seealso{the main function \code{\link{cobs}}; further
  \code{\link{drqssbc}} which is called from \code{qbsks()}.
}
\examples{
}
\keyword{smooth}
\keyword{utilities}
\eof
